# 函数高级

## 函数的默认参数

---

在C++中，函数的形参列表中，形参也可以有默认值。

`返回值类型 函数名 (参数 = 默认值) {}`

``` c++ linenums="1" title="示例"
#include <iostream>
using namespace std;

int function (int a = 10, int b = 20, int c = 30){
    return a + b + c;
}

int main(){
    cout << function() << endl;
    system("pause");
    return 0;
}
```

<br>

* **注意**

如果函数的形参中某个位置已经有了默认参数，那么此位置后的所有形参都必须有默认参数。

`int function (int a, int b = 10, int c) {}` 这样的函数就是不合法的。

* **注意**

如果函数的声明中有了默认参数，那么函数的实现中就不能有默认参数。

同理，在函数的声明和实现中只能有一个有默认参数。

``` c++ linenums="1" title="错误示例"
#include <iostream>
using namespace std;

// 函数声明
int function (int a = 10, int b = 20);

// 函数实现
int function (int a = 10. int b = 20){
  return a + b + c;
}

int main (){
  cout << function() << endl;
  system("pause");
  return 0;
}
```

## 函数的占位参数

---

C++中函数的形参列表中可以有占位参数，用于作占位，调用函数时必须填补该位置。

占位参数也可以有默认值。

`返回值类型 函数名 (数据类型) {}`

``` c++ linenums="1" title="示例"
#include <iostream>
using namespace std;

void function (int a = 10, int){
    cout << "this is a function" << endl;
}

int main(){
    function(10,10);
    system("pause");
    return 0;
}
```

## 函数重载

### 函数重载概述

---

C++中的函数重载的作用时使函数名可以相同，提高复用性。

使用函数重载需要满足的条件：

* 同一个作用域下
* 函数名相同
* 函数参数类型不同、个数不同、顺序不同

另外，函数的返回值不可以成为函数重载的满足条件。

``` c++ linenums="1" title="示例"
#include <iostream>
using namespace std;

void function (int a){
    cout << "function-Alpha" << endl;
}

void function (const int a){
    cout << "function-Barvo" << endl;
}

int main(){

    int a = 10;
    const int b = 10;

    function(a);

    function(b);

    system("pause");
    return 0;
}
```

> 上述程序中，第一个调用的function输出的是“**function-Alpha**”，而第二个function输出的是“**function-Barvo**”;这个函数重载中调用哪一个函数取决于传入参数的类型。

### 引用作为函数重载条件

---

在函数重载中，引用也可以作为函数重载的条件。

``` c++ linenums="1" title="示例代码"
#include <iostream>
using namespace std;

void function (int &a){
    cout << "function-Alpha" << endl;
}

void function (const int &a){
    cout << "function-Barvo" << endl;
}

int main(){

    int a = 10;
    const int b = 10;

    function(a);

    function(b);

    system("pause");
    return 0;
}
```

### 函数重载与默认参数

---

在同时使用函数重载和默认参数时有可能会产的代码逻辑上的二义性。

尽量避免在使用函数重载的时候使用默认参数。

``` c++ linenums="1" title="示例"
#include <iostream>
using namespace std;

void function (int a, int b = 10){
  cout << "function-Alpha" << endl;
}

void function (int a){
  cout << "function-Alpha" << endl;
}

int main (){

  // 此时代码就出现了二义性，其传入参数既可以满足第一个函数，又可以满足第二个函数
  function (10);

  system("pause");
  return 0;
}

```