## 整形

**作用**：表示整形类型的数据。

| 数据类型              | 占用空间                                            | 取值范围       |
| --------------------- | --------------------------------------------------- | -------------- |
| short（短整型)        | 2字节                                               | -2^15 ~ 2^15-1 |
| int（整形）           | 4字节                                               | -2^31 ~ 2^31-1 |
| long（长整形）        | Windows：4字节、Linux：4字节（32位）、8字节（64位） | -2^31 ~ 2^31-1 |
| long long（长长整形） | 8字节                                               | -2^63 ~ 2^63-1 |

## 实型（浮点型）

**作用**：用于表示小数。

浮点型变量分为两种：

- 单精度浮点型：float
- 双精度浮点型：double

二者的区别是表示的数字范围不同，在C++中：有效数字指的是整数位和小数位的和。

| 数据类型 | 占用空间 | 有效范围        |
| -------- | -------- | --------------- |
| float    | 4字节    | 7位有效数字     |
| double   | 8字节    | 15~16位有效数字 |

**示例**：

``` c++ linenums="1"
int main(){
	
    float f1 = 3.14f;
    double d1 = 3.14;
    
    cout << f1 << endl;
    cout << d1 << endl;
    
    system("pause");
    return 0;
}
```

## 字符型

**作用**：用于显示**单个**字符。

**语法**：`char ch = 'a'`

> 在显示字符型变量时，用单引号将字符括起来
>
> 单引号内只能有一个字符
>
> C和C++中，字符型变量只占用一个字节
>
> 字符型变量并不是吧字符本身放到内存中，而是存储对应的ASCII码值

**示例**：

``` c++ linenums="1"
int main(){
    char ch = 'a';
    cout << ch << endl;
    cout << sizeof(char) << endl;
    
    system ("pause");
    return 0;
}
```

## 转义字符

**作用**：用于表示一些ASCII不能表示出来的字符。

| 转移字符 | 含义                                   | ASCII码值（十进制） |
| -------- | -------------------------------------- | ------------------- |
| \a       | 警报                                   | 007                 |
| \b       | 退格（BS），将当前位置移动到第一列     | 008                 |
| \f       | 换页（FF），将当前位置移动到下页开头   | 012                 |
| \n       | 换行（LF），将当前位置移动到下一行开头 | 010                 |
| \r       | 回车（CR），将当前位置跳到本行开头     | 013                 |
| \t       | 水平制表（HT），跳到下一个Tab的位置    | 009                 |
| \v       | 垂直制表（VT）                         | 011                 |
| \\\      | 代表一个反斜线字符“**\\**”             | 092                 |
| \\'      | 代表一个单引号字符                     | 039                 |
| \\"      | 代表一个双引号字符                     | 034                 |
| \?       | 代表一个问号                           | 063                 |

## 字符串型

**作用**：用于表示一串字符。

C++中有两种风格的字符串：

* **C风格的字符串**：`char 变量名[]  = "字符串值"`
  * **示例**：

``` c++ linenums="1"
int main(){
    char str1[] = "hello world";
    cout << str1 << endl;
    
    system ("pause");
    return 0;
}
```

* **C++风格的字符串**：`string 变量名 = "字符串值"`
  * **示例**：

``` c++ linenums="1"
int main(){
	string str2 = "hello world";
    cout << str2 << endl;
    
    system ("pause");
    return 0;
}
```

## 布尔型

作用：**代表真或者假的值**。

bool类型只有两个值：

* True——真（非零）
* False——假 （零）

bool类型占**1个字节**大小。

**示例**：

``` c++ linenums="1"
int main(){
	bool flag = true;
    cout << flag << endl;
    
    flah = false;
    cout << flag << end;
    
    cout << "size of bool  =" << sizeof(bool) << endl;
    
    system("pause");
    return 0;
}
```

## 数据的输入

**作用**：用于从键盘获取数据。

**关键字**：**cin**

**语法**：`cin >> 变量`

**示例**：

``` c++ linenums="1"
int main(){
	//整形输入
    int a = 0;
    cout << "请输入整形变量：" << endl;
    cin >> a;
    cout << a << endl;
    
    //浮点型输入
    double b = 0;
    cout << "请输入浮点型变量：" << endl;
    cin >> b;
    cout << b << endl;
   	
    //字符型输入
    char ch = 0;
    cout << "请输入字符型变量：" << endl;
    cin >> ch;
    cout << ch << endl;
    
    //字符串输入
    string str = "h";
    cout << "请输入一个字符串：" << end;
    cin >> str;
    cout << str << endl;
    
    //布尔类型
    bool flag = true;
    cout << "请输入一个Bool：" << endl;
    cin >> flag;
    cout << flag << endl;
    
	system("pause");
    return 0;
}
```

## sizeof关键字

**作用**：利用**sizeof**关键字可以统计数据类型所占内存大小。

**语法**：`sizeof( 数据 \ 变量 )`

**实例**：

``` c++ linenums="1"  
int main(){
	cout << "short 类型所占内存空间为：" << sizeof(short) << endl;
    
    cout << "int 类型所占内存空间为：" << sizeof(int) << endl;
    
    cout << "long 类型所占内存空间为：" << sizeof(long long) << endl;
    
    cout << "long long 类型所占内存空间为：" << sizeof(long long) << endl;
    
    system("pause");
    return 0;
}
```